module pcl_module

!***********************************************************************
!
!     Fortran-definitions for PCL (Performance Counter Library)
!
!                  PCL - The Performance Counter Library
!
!
!   Copyright (C) 1999, Forschungszentrum Juelich GmbH, Federal Republic of
!   Germany. All rights reserved.
!
!   Redistribution and use in source and binary forms, with or without
!   modification, are permitted provided that the following conditions are met:
!
!   Redistributions of source code must retain the above copyright notice, this
!   list of conditions and the following disclaimer.
!
!     - Redistributions of source code must retain the above copyright notice,
!       this list of conditions and the following disclaimer.
!
!     - Redistributions in binary form must reproduce the above copyright
!       notice, this list of conditions and the following disclaimer in the
!       documentation and/or other materials provided with the distribution.
!
!     - Any publications that result from the use of this software shall
!       reasonably refer to the Research Centre's development.
!
!     - All advertising materials mentioning features or use of this software
!       must display the following acknowledgement:
!
!           This product includes software developed by Forschungszentrum
!           Juelich GmbH, Federal Republic of Germany.
!
!     - Forschungszentrum Juelich GmbH is not obligated to provide the user with
!       any support, consulting, training or assistance of any kind with regard
!       to the use, operation and performance of this software or to provide
!       the user with any updates, revisions or new versions.
!
!
!   THIS SOFTWARE IS PROVIDED BY FORSCHUNGSZENTRUM JUELICH GMBH "AS IS" AND ANY
!   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
!   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
!   DISCLAIMED. IN NO EVENT SHALL FORSCHUNGSZENTRUM JUELICH GMBH BE LIABLE FOR
!   ANY SPECIAL, DIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
!   RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
!   CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
!   CONNECTION WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE.C
!
!***********************************************************************

implicit none

!*** interface functions
public :: pclquery, pclstart, pclread, pclstop 

!*** maximum values
integer, parameter, public :: PCL_MAX_EVENT = 61
integer, parameter, public :: PCL_MAX_EVENT_PER_CALL = 4
integer, parameter, public :: PCL_MAX_NESTING_LEVEL = 16
integer, parameter, public :: PCL_COUNTER_MAX = 4

!*** return codes
integer, parameter, public :: PCL_SUCCESS = 0
integer, parameter, public :: PCL_NOT_SUPPORTED = -1
integer, parameter, public :: PCL_TOO_MANY_EVENTS = -2
integer, parameter, public :: PCL_MANY_NESTINGS = -3
integer, parameter, public :: PCL_ILL_NESTING = -4
integer, parameter, public :: PCL_ILL_EVENT = -5
integer, parameter, public :: PCL_MODE_NOT_SUPPORTED = -6
integer, parameter, public :: PCL_FAILURE = -7

!*** define what to count
integer, parameter, public :: PCL_MODE_USER = 1
integer, parameter, public :: PCL_MODE_SYSTEM = 2
integer, parameter, public :: PCL_MODE_USER_SYSTEM = 3

!*** events
integer, parameter, public :: PCL_L1CACHE_READ = 0
integer, parameter, public :: PCL_L1CACHE_WRITE = 1
integer, parameter, public :: PCL_L1CACHE_READWRITE = 2
integer, parameter, public :: PCL_L1CACHE_HIT = 3
integer, parameter, public :: PCL_L1CACHE_MISS = 4
integer, parameter, public :: PCL_L1DCACHE_READ = 5
integer, parameter, public :: PCL_L1DCACHE_WRITE = 6
integer, parameter, public :: PCL_L1DCACHE_READWRITE = 7
integer, parameter, public :: PCL_L1DCACHE_HIT = 8
integer, parameter, public :: PCL_L1DCACHE_MISS = 9
integer, parameter, public :: PCL_L1ICACHE_READ = 10
integer, parameter, public :: PCL_L1ICACHE_WRITE = 11
integer, parameter, public :: PCL_L1ICACHE_READWRITE = 12
integer, parameter, public :: PCL_L1ICACHE_HIT = 13
integer, parameter, public :: PCL_L1ICACHE_MISS = 14
integer, parameter, public :: PCL_L2CACHE_READ = 15
integer, parameter, public :: PCL_L2CACHE_WRITE = 16
integer, parameter, public :: PCL_L2CACHE_READWRITE = 17
integer, parameter, public :: PCL_L2CACHE_HIT = 18
integer, parameter, public :: PCL_L2CACHE_MISS = 19
integer, parameter, public :: PCL_L2DCACHE_READ = 20
integer, parameter, public :: PCL_L2DCACHE_WRITE = 21
integer, parameter, public :: PCL_L2DCACHE_READWRITE = 22
integer, parameter, public :: PCL_L2DCACHE_HIT = 23
integer, parameter, public :: PCL_L2DCACHE_MISS = 24
integer, parameter, public :: PCL_L2ICACHE_READ = 25
integer, parameter, public :: PCL_L2ICACHE_WRITE = 26
integer, parameter, public :: PCL_L2ICACHE_READWRITE = 27
integer, parameter, public :: PCL_L2ICACHE_HIT = 28
integer, parameter, public :: PCL_L2ICACHE_MISS = 29
integer, parameter, public :: PCL_TLB_HIT = 30
integer, parameter, public :: PCL_TLB_MISS = 31
integer, parameter, public :: PCL_ITLB_HIT = 32
integer, parameter, public :: PCL_ITLB_MISS = 33
integer, parameter, public :: PCL_DTLB_HIT = 44
integer, parameter, public :: PCL_DTLB_MISS = 35
integer, parameter, public :: PCL_CYCLES = 36
integer, parameter, public :: PCL_ELAPSED_CYCLES = 37
integer, parameter, public :: PCL_INTEGER_INSTR = 38
integer, parameter, public :: PCL_FP_INSTR = 39
integer, parameter, public :: PCL_LOAD_INSTR = 40
integer, parameter, public :: PCL_STORE_INSTR = 41
integer, parameter, public :: PCL_LOADSTORE_INSTR = 42
integer, parameter, public :: PCL_INSTR = 43
integer, parameter, public :: PCL_JUMP_SUCCESS = 44
integer, parameter, public :: PCL_JUMP_UNSUCCESS = 45
integer, parameter, public :: PCL_JUMP = 46
integer, parameter, public :: PCL_ATOMIC_SUCCESS = 47
integer, parameter, public :: PCL_ATOMIC_UNSUCCESS = 48
integer, parameter, public :: PCL_ATOMIC = 49
integer, parameter, public :: PCL_STALL_INTEGER = 50
integer, parameter, public :: PCL_STALL_FP = 51
integer, parameter, public :: PCL_STALL_JUMP = 52
integer, parameter, public :: PCL_STALL_LOAD = 53
integer, parameter, public :: PCL_STALL_STORE = 54
integer, parameter, public :: PCL_STALL = 55
integer, parameter, public :: PCL_MFLOPS = 56
integer, parameter, public :: PCL_IPC = 57
integer, parameter, public :: PCL_L1DCACHE_MISSRATE = 58
integer, parameter, public :: PCL_L2DCACHE_MISSRATE = 59
integer, parameter, public :: PCL_MEM_FP_RATIO = 60

!*** interface functions

! query for functionality.
! returns PCL_SUCCESS if requested event counting is possible
interface pclquery
   function pclquery(counter_list, ncounter, flags)
     integer, dimension(ncounter), intent(in)  :: counter_list ! I: requested event counters
     integer,                      intent(in)  :: ncounter     ! I: number of requested event counter
     integer,                      intent(in)  :: flags        ! I: mode flags (PCL_MODE_xxx)
     integer                                   :: pclquery
   end function pclquery
end interface

! start performance counting
! returns PCL_SUCCESS if everything went ok
interface pclstart
   function pclstart(counter_list, ncounter, flags)
     integer, dimension(ncounter), intent(in)  :: counter_list ! I: event counters to be started
     integer,                      intent(in)  :: ncounter     ! I: number of event counters
     integer,                      intent(in)  :: flags        ! I: flags (PCL_MODE_xxx)
     integer                                   :: pclstart
   end function pclstart
end interface 

! reads performance counters.
! result_list must be a pointer to a sufficiently large buffer.
! returns PCL_SUCCESS if everything went ok
interface pclread
   function pclread(i_result_list, fp_result_list, ncounter)
     integer, parameter                                  :: i64 = selected_int_kind(18) ! 64 Bit
     integer, parameter                                  :: r64 = selected_real_kind(13)! 64 Bit
     integer(kind=i64), dimension(ncounter), intent(out) :: i_result_list  ! O: integer counter values
     real(kind=r64), dimension(ncounter), intent(out)    :: fp_result_list ! O: FP counter values
     integer,                                intent(in)  :: ncounter       ! I: number of event counters
     integer                                             :: pclread
   end function pclread
end interface

! stops performance counting and reads counters values.
! result_list must be a pointer to a sufficiently large buffer.
! returns PCL_SUCCESS if everything went ok
interface pclstop
   
   function pclstop(results_list, ncounter)
     integer, parameter                                  :: i64 = selected_int_kind(18) ! 64 Bit
     integer, parameter                                  :: r64 = selected_real_kind(13)! 64 Bit
     integer(kind=i64), dimension(ncounter), intent(out) :: i_result_list  ! O: integer counter values
     real(kind=r64), dimension(ncounter), intent(out)    :: fp_result_list ! O: FP counter values
     integer,                                 intent(in) :: ncounter       ! I: number of event counters
     integer                                             :: pclstop
   end function pclstop
end interface

!***********************************************************************

end module
